==> libtunepimp-0.2.0.tar.gz: libtunepimp-0.2.0/lib/metadata.cpp <==
/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: metadata.cpp,v 1.6 2003/06/24 22:57:28 robert Exp $

----------------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include "metadata.h"
#include "../config.h"

extern "C"
{
   #include "fstrcmp.h"
}

const char *albumTypeStrings[] =
{
    "album", "single", "compilation", "soundtrack",
    "spokenword", "audiobook", "live", "other", "error", "\0"
};

const char *albumStatusStrings[] =
{
    "official", "promotion", "bootleg", "\0"
};

TPAlbumType convertToAlbumType(const char *albumType)
{
    for(int i = 0;; i++)
    {
        if (albumTypeStrings[i][0] == 0)
            break;

        if (strlen(albumType) > 4 && strcasecmp(albumTypeStrings[i], albumType + 4) == 0)
            return (TPAlbumType)i;
    }
    return eAlbumType_Error;
}

void convertFromAlbumType(TPAlbumType type, string &albumType)
{
    albumType = string(albumTypeStrings[(int)type]);
}

TPAlbumStatus convertToAlbumStatus(const char *albumStatus)
{
    for(int i = 0;; i++)
    {
        if (albumStatusStrings[i][0] == 0)
            break;

        if (strlen(albumStatus) > 6 && strcasecmp(albumStatusStrings[i], albumStatus + 6) == 0)
            return (TPAlbumStatus)i;
    }
    return eAlbumStatus_Error;
}

void convertFromAlbumStatus(TPAlbumStatus status, string &albumStatus)
{
    albumStatus = string(albumStatusStrings[(int)status]);
}

struct Weights
{
    double artist;
    double album;
    double track;
    double trackNum;
    double duration;
};

const Weights weights[32] =
{
//    ar    al    tr    tn     du         ar  al  tr  tn  du
    { 0.00, 0.00, 0.00, 0.00,  0.00 }, //  0   0   0   0   0
    { 0.00, 0.00, 0.00, 0.00,  0.95 }, //  0   0   0   0   1
    { 0.00, 0.00, 0.00, 0.95,  0.00 }, //  0   0   0   1   0
    { 0.00, 0.00, 0.00, 0.25,  0.70 }, //  0   0   0   1   1
    { 0.00, 0.00, 0.95, 0.00,  0.00 }, //  0   0   1   0   0
    { 0.00, 0.00, 0.75, 0.00,  0.20 }, //  0   0   1   0   1
    { 0.00, 0.00, 0.75, 0.20,  0.00 }, //  0   0   1   1   0
    { 0.00, 0.00, 0.65, 0.10,  0.20 }, //  0   0   1   1   1
    { 0.00, 1.00, 0.00, 0.00,  0.00 }, //  0   1   0   0   0
    { 0.00, 0.80, 0.00, 0.00,  0.20 }, //  0   1   0   0   1
    { 0.00, 0.80, 0.00, 0.20,  0.00 }, //  0   1   0   1   0
    { 0.00, 0.70, 0.00, 0.10,  0.20 }, //  0   1   0   1   1
    { 0.00, 0.50, 0.50, 0.00,  0.00 }, //  0   1   1   0   0
    { 0.00, 0.40, 0.40, 0.00,  0.20 }, //  0   1   1   0   1
    { 0.00, 0.45, 0.45, 0.10,  0.00 }, //  0   1   1   1   0
    { 0.00, 0.35, 0.35, 0.15,  0.15 }, //  0   1   1   1   1

    { 0.95, 0.00, 0.00, 0.00,  0.00 }, //  1   0   0   0   0
    { 0.75, 0.00, 0.00, 0.00,  0.20 }, //  1   0   0   0   1
    { 0.85, 0.00, 0.00, 0.10,  0.00 }, //  1   0   0   1   0
    { 0.60, 0.00, 0.00, 0.10,  0.25 }, //  1   0   0   1   1
    { 0.48, 0.00, 0.47, 0.00,  0.00 }, //  1   0   1   0   0
    { 0.43, 0.00, 0.42, 0.00,  0.10 }, //  1   0   1   0   1
    { 0.43, 0.00, 0.42, 0.10,  0.00 }, //  1   0   1   1   0
    { 0.38, 0.00, 0.37, 0.10,  0.10 }, //  1   0   1   1   1
    { 0.50, 0.50, 0.00, 0.00,  0.00 }, //  1   1   0   0   0
    { 0.45, 0.45, 0.00, 0.00,  0.10 }, //  1   1   0   0   1
    { 0.45, 0.45, 0.00, 0.10,  0.00 }, //  1   1   0   1   0
    { 0.40, 0.40, 0.00, 0.10,  0.10 }, //  1   1   0   1   1
    { 0.33, 0.33, 0.34, 0.00,  0.00 }, //  1   1   1   0   0
    { 0.30, 0.30, 0.30, 0.00,  0.10 }, //  1   1   1   0   1
    { 0.30, 0.30, 0.30, 0.10,  0.00 }, //  1   1   1   1   0
    { 0.25, 0.25, 0.25, 0.125, 0.125}  //  1   1   1   1   1
};

double MetadataCompare::durationSim(int trackA, int trackB) const
{
    int diff;

    diff = abs(trackA - trackB);
    if (diff > 30000)
       return 0;

    return 1.0 - ((double)diff / (double)30000);
}

int MetadataCompare::compare(const Metadata &a, const Metadata &b) const
{
    int index = 0;
    Metadata A = a, B = b;

    // If one of the two is completely empty of meaningful info, just return 0
    if ((A.artist.empty() && A.album.empty() && A.track.empty()) ||
        (B.artist.empty() && B.album.empty() && B.track.empty()))
        return 0;

    if (!A.artist.empty() && !B.artist.empty())
        index |= 16;

    // If one album is blank, and the other is an album, copy it over to favor it.
    if (A.album.empty() && !B.album.empty() && B.albumType == eAlbumType_Album)
        A.album = B.album;

    // Now check the reverse case as well.
    if (B.album.empty() && !A.album.empty() && A.albumType == eAlbumType_Album)
        B.album = A.album;

    if (!A.album.empty() && !B.album.empty())
        index |= 8;

    if (!A.track.empty() && !B.track.empty())
        index |= 4;

    if (A.trackNum != 0 && B.trackNum != 0)
        index |= 2;

    if (A.duration != 0 && B.duration != 0)
        index |= 1;

    if (index == 0)
        return 0;

    return (int)ceil(((fstrcmp(A.artist.c_str(), B.artist.c_str(), .0) * weights[index].artist +
                       fstrcmp(A.album.c_str(), B.album.c_str(), .0) * weights[index].album +
                       fstrcmp(A.track.c_str(), B.track.c_str(), .0) * weights[index].track +
                       durationSim(A.duration, B.duration) * weights[index].duration +
                       ((A.trackNum == B.trackNum) ? 1.0 : 0.0) * weights[index].trackNum) * 100));
}

==> libtunepimp-0.3.0.tar.gz: libtunepimp-0.3.0/lib/metadata.cpp <==
/*----------------------------------------------------------------------------

   libtunepimp -- The MusicBrainz tagging library.  
                  Let a thousand taggers bloom!
   
   Copyright (C) Robert Kaye 2003
   
   This file is part of libtunepimp.

   libtunepimp is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   libtunepimp is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with libtunepimp; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   $Id: metadata.cpp,v 1.11 2004/03/02 08:38:52 robert Exp $

----------------------------------------------------------------------------*/
#include <math.h>
#include <stdio.h>
#include "metadata.h"
#include "../config.h"

extern "C"
{
   #include "fstrcmp.h"
}

const int numAlbumTypeStrings = 11;
const char *albumTypeStrings[] =
{
    "album", "single", "EP", "compilation", "soundtrack",
    "spokenword", "interview", "audiobook", "live", "remix", "other", "\0"
};

const int numAlbumStatusStrings = 3;
const char *albumStatusStrings[] =
{
    "official", "promotion", "bootleg", "\0"
};

TPAlbumType convertToAlbumType(const char *albumType)
{
    for(int i = 0;; i++)
    {
        if (albumTypeStrings[i][0] == 0)
            break;

        if (strlen(albumType) > 4 && strcasecmp(albumTypeStrings[i], albumType + 4) == 0)
            return (TPAlbumType)i;

        if (strcasecmp(albumTypeStrings[i], albumType) == 0)
            return (TPAlbumType)i;
    }
    return eAlbumType_Error;
}

void convertFromAlbumType(TPAlbumType type, string &albumType)
{
    if ((int)type >= 0 && (int)type < numAlbumTypeStrings)
        albumType = string(albumTypeStrings[(int)type]);
    else 
        albumType = "unknown";
}

TPAlbumStatus convertToAlbumStatus(const char *albumStatus)
{
    for(int i = 0;; i++)
    {
        if (albumStatusStrings[i][0] == 0)
            break;

        if (strlen(albumStatus) > 6 && strcasecmp(albumStatusStrings[i], albumStatus + 6) == 0)
            return (TPAlbumStatus)i;

        if (strcasecmp(albumStatusStrings[i], albumStatus) == 0)
            return (TPAlbumStatus)i;
    }
    return eAlbumStatus_Error;
}

void convertFromAlbumStatus(TPAlbumStatus status, string &albumStatus)
{
    if ((int)status >= 0 && (int)status < numAlbumStatusStrings)
        albumStatus = string(albumStatusStrings[(int)status]);
    else 
        albumStatus = "unknown";
}

struct Weights
{
    double artist;
    double album;
    double track;
    double trackNum;
    double duration;
};

const Weights weights[32] =
{
//    ar    al    tr    tn     du         ar  al  tr  tn  du
    { 0.00, 0.00, 0.00, 0.00,  0.00 }, //  0   0   0   0   0
    { 0.00, 0.00, 0.00, 0.00,  0.95 }, //  0   0   0   0   1
    { 0.00, 0.00, 0.00, 0.95,  0.00 }, //  0   0   0   1   0
    { 0.00, 0.00, 0.00, 0.25,  0.70 }, //  0   0   0   1   1
    { 0.00, 0.00, 0.95, 0.00,  0.00 }, //  0   0   1   0   0
    { 0.00, 0.00, 0.75, 0.00,  0.20 }, //  0   0   1   0   1
    { 0.00, 0.00, 0.75, 0.20,  0.00 }, //  0   0   1   1   0
    { 0.00, 0.00, 0.65, 0.10,  0.20 }, //  0   0   1   1   1
    { 0.00, 1.00, 0.00, 0.00,  0.00 }, //  0   1   0   0   0
    { 0.00, 0.80, 0.00, 0.00,  0.20 }, //  0   1   0   0   1
    { 0.00, 0.80, 0.00, 0.20,  0.00 }, //  0   1   0   1   0
    { 0.00, 0.70, 0.00, 0.10,  0.20 }, //  0   1   0   1   1
    { 0.00, 0.50, 0.50, 0.00,  0.00 }, //  0   1   1   0   0
    { 0.00, 0.40, 0.40, 0.00,  0.20 }, //  0   1   1   0   1
    { 0.00, 0.45, 0.45, 0.10,  0.00 }, //  0   1   1   1   0
    { 0.00, 0.35, 0.35, 0.15,  0.15 }, //  0   1   1   1   1

    { 0.95, 0.00, 0.00, 0.00,  0.00 }, //  1   0   0   0   0
    { 0.75, 0.00, 0.00, 0.00,  0.20 }, //  1   0   0   0   1
    { 0.85, 0.00, 0.00, 0.10,  0.00 }, //  1   0   0   1   0
    { 0.60, 0.00, 0.00, 0.10,  0.25 }, //  1   0   0   1   1
    { 0.48, 0.00, 0.47, 0.00,  0.00 }, //  1   0   1   0   0
    { 0.43, 0.00, 0.42, 0.00,  0.10 }, //  1   0   1   0   1
    { 0.43, 0.00, 0.42, 0.10,  0.00 }, //  1   0   1   1   0
    { 0.38, 0.00, 0.37, 0.10,  0.10 }, //  1   0   1   1   1
    { 0.50, 0.50, 0.00, 0.00,  0.00 }, //  1   1   0   0   0
    { 0.45, 0.45, 0.00, 0.00,  0.10 }, //  1   1   0   0   1
    { 0.45, 0.45, 0.00, 0.10,  0.00 }, //  1   1   0   1   0
    { 0.40, 0.40, 0.00, 0.10,  0.10 }, //  1   1   0   1   1
    { 0.33, 0.33, 0.34, 0.00,  0.00 }, //  1   1   1   0   0
    { 0.30, 0.30, 0.30, 0.00,  0.10 }, //  1   1   1   0   1
    { 0.30, 0.30, 0.30, 0.10,  0.00 }, //  1   1   1   1   0
    { 0.25, 0.25, 0.25, 0.125, 0.125}  //  1   1   1   1   1
};

double MetadataCompare::durationSim(int trackA, int trackB) const
{
    int diff;

    diff = abs(trackA - trackB);
    if (diff > 30000)
       return 0;

    return 1.0 - ((double)diff / (double)30000);
}

int MetadataCompare::compare(const Metadata &a, const Metadata &b) const
{
    int index = 0;
    Metadata A = a, B = b;

    // If one of the two is completely empty of meaningful info, just return 0
    if ((A.artist.empty() && A.album.empty() && A.track.empty()) ||
        (B.artist.empty() && B.album.empty() && B.track.empty()))
        return 0;

    if (!A.artist.empty() && !B.artist.empty())
        index |= 16;

    // If one album is blank, and the other is an album, copy it over to favor it.
    if (A.album.empty() && !B.album.empty() && B.albumType == eAlbumType_Album)
        A.album = B.album;

    // Now check the reverse case as well.
    if (B.album.empty() && !A.album.empty() && A.albumType == eAlbumType_Album)
        B.album = A.album;

    if (!A.album.empty() && !B.album.empty())
        index |= 8;

    if (!A.track.empty() && !B.track.empty())
        index |= 4;

    if (A.trackNum != 0 && B.trackNum != 0)
        index |= 2;

    if (A.duration != 0 && B.duration != 0)
        index |= 1;

    if (index == 0)
        return 0;

    return (int)ceil(((fstrcmp(A.artist.c_str(), B.artist.c_str(), .0) * weights[index].artist +
                       fstrcmp(A.album.c_str(), B.album.c_str(), .0) * weights[index].album +
                       fstrcmp(A.track.c_str(), B.track.c_str(), .0) * weights[index].track +
                       durationSim(A.duration, B.duration) * weights[index].duration +
                       ((A.trackNum == B.trackNum) ? 1.0 : 0.0) * weights[index].trackNum) * 100));
}
